#!BPY

"""
Name: 'The Movies (.msh)...'
Blender: 243
Group: 'Export'
Tooltip: 'Export to Lionhead Studios The Movies file format. (.msh)'
"""

__author__ = ["Mark S Andrews"]
__url__ = ("http://themovieseditor.com","http://tmws.themoviesplanet.com")
__version__ = "0.15"
__bpydoc__ = """\

Msh Exporter

This script exports a msh file for use with Lionhead Studio Limited's
The Movies. It is not supported by either Lionhead or Activision.

It does not currently support all features of the msh files or even
all features currently imported. It is suitable for static props only.

Known Problems:

Submeshes, Rooms, Weights and Bones are not exported correctly.

Not all flags are supported currently.

Changes:

0.15 Nick Hudson 2007-03-11<br>
- Better handling of odd uv-mapped vertices
0.14 Nick Hudson 2007-03-09<br>
- Fixes for hand-helds
0.13 Nick Hudson 2007-03-09<br>
- Changes for hand-helds
0.12 Nick Hudson 2007-03-09<br>
- Fix setting of boolean has_bones
0.11 Nick Hudson 2007-03-09<br>
- Change sort order of weight per vertex, according to descending weight strength.
0.10 Nick Hudson 2007-03-09<br>
- Changed the order of the export to put the head as the last bone
0.09 Nick Hudson 2007-03-08<br>
- Change to use a Matrix for bone head/tail
0.08 Nick Hudson 2007-03-08<br>
- Fixed bug with bone/weight export. Removed group_names as it was intefering with bones.
0.07 Nick Hudson 2007-03-08<br>
- More work with bones
0.06 Nick Hudson 2007-03-08<br>
- Lots of work with bones/weights. Though the resulting meshes can't be imported yet, this is a lot closer to what we need, and may help with Gleem's rewrite of export (if he's doing it)
0.05 Nick Hudson 2007-01-15<br>
- Comment out getProperty line, which isn't to be used yet.
0.04 Nick Hudson 2007-01-12<br>
- Add reference to tmmesh, and add changes for hand held props.
0.03 Nick Hudson 2007-01-08<br>
- Added basic logging, fixes for exporting sets.
0.02 Mark S Andrews 2006-01-29<br>
- First working version, various fixes
0.01 Mark S Andrews 2006-01-13<br>
- Initial version

"""

# ***** BEGIN GPL LICENSE BLOCK *****
#
# Script copyright (C) Mark S Andrews
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------


# Import modules

import Blender
from Blender import Image, Material, Texture, NMesh, Window, Armature
from Blender.Mathutils import Matrix, Vector
from random import getrandbits
import struct, os, re

# Placeholder for a proper mesh definition class.
from tmmesh import tmmesh

#***********************************************
# globals
#***********************************************

material_list = []
material_keys = []

#***********************************************
# helper classes & functions for msh file
#***********************************************

grouped_mesh = re.compile(r'(.+)\.(\d\d)\..+')
numbered_mesh = re.compile(r'(.+)\.\d+')
room_mesh = re.compile(r'(.+)\.room\_')

def write_nts(item, stream, length=32):
    if len(item) > length:
        print "ITEM: %s" % (item)
        print "LEN: %d" % (len(item))
        raise ValueError("%s is longer than %d"%(item, length))
    while len(item) < length:
        item += "\x00"
    stream.write(item)

def write_null(struct_type, stream):
    data = ""
    for b in range(struct.calcsize(struct_type)):
        data += "\x00"
    stream.write(data)


class range_adjust(object):
    def __init__(self, min, max):
        self.min = min
        self.max = max
        self.step = (max - min) / 65535
    def adjust(self, value):
        if self.step == 0:
            return 0
        return int((value - self.min) / self.step)

#***********************************************
# writer functions for msh file
#***********************************************

def write_material(mat, stream):
    image_list = Image.Get()
    texture_list = mat.getTextures()
    i = [-1,-1,-1,-1]
    use_alpha = False
    if texture_list[0]:
        im = texture_list[0].tex.getImage()
        if im:
            i[0] = image_list.index(im)
            use_alpha = texture_list[0].mtAlpha
    if texture_list[1]:
        im = texture_list[1].tex.getImage()
        if im:
            i[1] = image_list.index(im)
    if texture_list[2]:
        im = texture_list[2].tex.getImage()
        if im:
            i[3] = image_list.index(im)
    
    # *********** write image refs *************
    stream.write(struct.pack("bbbb", i[0], i[1], i[2], i[3]))
    # *********** too many unknowns!! **********
    stream.write("\x00\x01\x01")
    if use_alpha:
        stream.write("\x01")
    else:
        stream.write("\x00")
    write_null("BBBBBB", stream)
    stream.write("\x04\x00\xee\xa2\x00\xff")
    write_null("BBBB", stream)


#***********************************************
# Write the control meshes in the correct order.
#***********************************************
def write_control_meshes(stream):
    '''This sets the order for writing out a control mesh, since
       we are not sure what order these meshes may come in'''
    if binding is not None:
        write_basic_mesh(binding, stream)

    if floor is not None:
        write_basic_mesh(floor, stream)

    if wall is not None:
        write_basic_mesh(wall, stream)

    if land is not None:
        write_basic_mesh(land, stream)

    if mesh1 is not None:
        write_basic_mesh(mesh1, stream)

    if foundation is not None:
        write_basic_mesh(foundation, stream)

    if mesh2 is not None:
        write_basic_mesh(mesh2, stream)



def write_basic_mesh(mesh, stream):
    '''This is intended for writing out control meshes only, which
       require less data.'''

    meshdata = mesh.getData()
    meshlog.write('\n> write_basic_mesh: "%s"' % meshdata.name)
    
    # ********** write vertex count ************
    stream.write(struct.pack("L",len(meshdata.verts)))
    # stream.write(len(mesh.verts))
    # ********** write face count **************
    stream.write(struct.pack("L",len(meshdata.faces)))
    #stream.write(len(mesh.faces))
    for v in meshdata.verts:
        # ******* write vertex position ********
        stream.write(struct.pack("fff", v.co[0], v.co[1], v.co[2]))
    for f in meshdata.faces:
        v0 = meshdata.verts.index(f.v[0])
        v1 = meshdata.verts.index(f.v[1])
        v2 = meshdata.verts.index(f.v[2])
        # ******** write face vertices *********
        stream.write(struct.pack("HHH", v0, v1, v2))
    if len(meshdata.faces) & 1:
        # ********** write pad bytes ***********
        stream.write("\x00\x00")

def write_group(meshes, stream):
    # A thread on TMWS suggests that it may not be a matrix. So... 
    # this might need to be changed.

    matrix = meshes[0].matrix
    #matrix = [[1,0,0,0],[0,1,0,0],[0,0,1,0],[0,0,0,1]]
    flags = 16
    # *********** write group header ***********
    stream.write(struct.pack("BBBB", len(meshes), 0, 0, flags))
    # ************** write matrix **************
    for p in range(4):
        stream.write(struct.pack("fff", matrix[0][p], \
            matrix[1][p], matrix[2][p]))
    
    if flags & 16:
        write_null("L", stream)
        
    for m in meshes:
        write_mesh(m, stream)

def write_mesh(ob, stream):
    # this is the main function for writing a mesh to file. This is for
    # normal meshes only, and should not be used for control meshes 
    # (which don't have textures anyway

    global material_keys
    mesh = ob.getData()
    meshlog.write('\n> write_mesh: "%s"' % mesh.name)
    mat = mesh.getMaterials()
    # ********** write material index **********
    if mat:
        stream.write(struct.pack("L",material_keys.index(mat[0].name)))
    else:
        write_null("L", stream)
    meshlog.write('\n> mesh faces: "%s"' % len(mesh.faces))
    # ********** write face count **************
    stream.write(struct.pack("L",len(mesh.faces)))

    spos = stream.tell()
    meshlog.write('\n> mesh verts: "%s" at %d' % (len(mesh.verts), spos))
    # ********** write vertex count ************
    vcpos = stream.tell()
    stream.write(struct.pack("L",len(mesh.verts)))
    mf = [0x70,0,0,0]
    if mesh.getVertGroupNames() != []:
        mf[0] |= 1
    meshlog.write('\n> mesh_flags: "%s"' % mesh.name)
    # ************ write mesh flags ************
    stream.write(struct.pack("BBBB", mf[0], mf[1], mf[2], mf[3]))
    stream.write(struct.pack("L", getrandbits(32)))
    stream.write("\xff\xff\xff\xff")
    write_null("L", stream)
    xs = []
    ys =[]
    zs = []
    txs = []
    tys = []
    for v in mesh.verts:
        xs.append(v.co[0])
        ys.append(v.co[1])
        zs.append(v.co[2])
    for f in mesh.faces:
        for vertexid in range(3):
            txs.append(f.uv[vertexid][0])
            tys.append(f.uv[vertexid][1])

    meshlog.write('\n> mesh ranges: "%s"' % mesh.name)
    xr = range_adjust(min(xs), max(xs))
    yr = range_adjust(min(ys), max(ys))
    zr = range_adjust(min(zs), max(zs))
    nr = range_adjust(-1.0, 1.0)
    txr = range_adjust(min(txs), max(txs))
    tyr = range_adjust(min(tys), max(tys))
    # ************** write ranges *************
    stream.write(struct.pack("f", xr.min))
    stream.write(struct.pack("f", yr.min))
    stream.write(struct.pack("f", zr.min))
    stream.write(struct.pack("f", xr.max))
    stream.write(struct.pack("f", yr.max))
    stream.write(struct.pack("f", zr.max))
    stream.write(struct.pack("f", txr.min))
    stream.write(struct.pack("f", 1 - min(tys)))
    stream.write(struct.pack("f", txr.max))
    stream.write(struct.pack("f", 1 - max(tys)))
    # ************ write unknown4 *************
    write_null("BBBBBBBBBBBBBBBB", stream)
    # *********** write switch count **********
    write_null("L", stream)
    del (xs, ys, zs, txs, tys)
    verts = []
    vtrans = []
    for f in mesh.faces:
        if len(f.v) != 3:
            stream.close()
            raise ValueError("Incorrect number of vertices. Only triangles are supported.")
        for vertexid in range(3):
            v = f.v[vertexid]
            cv = []
            cv.append(xr.adjust(v.co[0]))
            cv.append(yr.adjust(v.co[1]))
            cv.append(zr.adjust(v.co[2]))
            cv.append(nr.adjust(v.no[0]))
            cv.append(nr.adjust(v.no[1]))
            cv.append(nr.adjust(v.no[2]))
            cv.append(txr.adjust(f.uv[vertexid][0]))
            cv.append(tyr.adjust(f.uv[vertexid][1]))

            # There was an issue where uv-mapping on some faces meant that
            # a single vertex could have two different 'cv' translations, 
            # which meant that the script was guessing we had more verts than
            # we did.
            # Changed method to ensure we always end up with the right number
            # of vertices.
            vref = mesh.verts.index(v)
            if vref in vtrans:
                newid = vtrans.index(vref)
            else:
                verts.append(cv)                    
                vtrans.append(vref)                  
                newid = len(vtrans) - 1

            #if cv in verts:
            #    newid = verts.index(cv)
            #else:
            #    verts.append(cv)
            #    newid = len(verts) - 1
            # ****** write face vertexid *******
            stream.write(struct.pack("H", newid))
            meshlog.write('\n> Face Vertex: "%s"' % (newid))
    if len(mesh.faces) & 1:
        stream.write("\x00\x00")
    vpos = stream.tell()
    stream.seek(vcpos)
    stream.write(struct.pack("L", len(vtrans)))
    spos = stream.tell()
    meshlog.write('\n> len(vtrans): "%d" at %d' % (len(vtrans), spos))
    stream.seek(vpos)

    meshlog.write('\n> Lightmaps')
    for v in verts:
        # ********** write vertex **************
        stream.write(struct.pack("HHHHHHHH", \
            v[0], v[1], v[2], v[3], v[4], v[5], v[6], v[7]))

    # write-out for lightmap should go in here, before weights

    meshlog.write('\n> Has_Bones? "%s"' % (has_bones))
    spos = stream.tell()
    meshlog.write('\n>> Marker Pre-Bone Weights: "%s"' % (spos))
    if has_bones:
        bw_lookup = []
        rootbone = Armature.Get("Bones").bones["root"]
        headbone = Armature.Get("Bones").bones["head"]
        bonekeys = rootbone.getAllChildren()
        bonekeys.insert(0, rootbone)
        meshlog.write('\n> Bone Keys for Verts: "%s"' % (bonekeys))
        for bone in bonekeys:
            bw_lookup.append(bone.name)
        bw_lookup.remove(headbone.name)
        bw_lookup.append(headbone.name)

        meshlog.write('\n> Bone List for Verts: "%s"' % (bw_lookup))
        meshlog.write('\n> Mesh: %s -- (Verts: %d)' % (mesh.name, len(mesh.verts)))

        # ********** write bone weights **************
        # w0,w1,w2,w3, b0,b1,b2,b3
        for v in mesh.verts:
            # fetch the bone/weight pair
            bw_pair = mesh.getVertexInfluences(v.index)
            bw_pair.sort(key=lambda x:-x[1])
            bonesort = []
            for bw in bw_pair:
                boneref = bw_lookup.index(bw[0])
                bonesort.append([bw[1], -boneref, boneref, bw[0]])
            bonesort.sort(lambda x,y: cmp(float(y[0]),float(x[0])) or \
                                         cmp(y[2],x[2]))

            bw_cnt = 0
            while bw_cnt < 4:
                if len(bw_pair) > bw_cnt:
                    stream.write(struct.pack("f", bonesort[bw_cnt][0]))
                else:
                    stream.write(struct.pack("f", 0))
                bw_cnt += 1

            bw_cnt = 0
            while bw_cnt < 4:
                if len(bw_pair) > bw_cnt:
                    last_bone = bonesort[bw_cnt][2]

                stream.write(struct.pack("b", last_bone))
                bw_cnt += 1

    spos = stream.tell()
    meshlog.write('\n>> Marker Post-Bone Weights: "%s"' % (spos))



def write_armature(stream):
    if has_bones:
        a = Armature.Get("Bones")
        bonelist = a.bones.values()
        if (len(bonelist)):
            meshlog.write('\n> bonelist: "%s"' % (bonelist))
            write_null("L", stream)
            meshlog.write('\n> Bone Count: "%d"' % (len(bonelist)))
            stream.write(struct.pack("L", len(bonelist)))
              
            rootbone = a.bones["root"]
            headbone = a.bones["head"]
            bonekeys = []
            for cbone in rootbone.getAllChildren():
                if cbone.name != "head":
                    bonekeys.append(cbone)
            bonekeys.insert(0, rootbone)
            bonekeys.append(headbone)

            bp_lookup = []
            meshlog.write('\n> bonekeys: "%s"' % (bonekeys))
            for bone in bonekeys:
                bp_lookup.append(bone.name)
                meshlog.write('\n> bone: "%s"' % (bone.name))
                boneparent = -1
                if bone.parent:
                    meshlog.write('\n> parent: "%s"' % (bone.parent.name))
                    meshlog.write('\n> pbone: "%s"' % (bone.parent))
                    
                    boneparent = bp_lookup.index(bone.parent.name)
                write_bone(stream, bone, boneparent)
            meshlog.write('\n> bp_lookup: "%s"' % (bp_lookup))

def write_bone(stream, bone, boneparent):
    write_nts(bone.name, stream)
    stream.write(struct.pack("l", boneparent))

    bone_matrix = bone.matrix["ARMATURESPACE"]

    stream.write(struct.pack("fff", bone_matrix[1][0], bone_matrix[1][1], bone_matrix[1][2]))
    stream.write(struct.pack("fff", bone_matrix[2][0], bone_matrix[2][1], bone_matrix[2][2]))
    stream.write(struct.pack("fff", bone_matrix[0][0], bone_matrix[0][1], bone_matrix[0][2]))
    stream.write(struct.pack("fff", bone_matrix[3][0], bone_matrix[3][1], bone_matrix[3][2]))
            
            

#***********************************************
# main export function
#***********************************************

def save_msh(filename):
    '''This is what writes to the msh file, or at least defines the order.
    '''
    # Am thinking we might want to move this around, so that we have a
    # proper class and such. There's no reason why we can't use coding
    # standards :)'''

    global material_list, material_keys, meshlog
    global binding, floor, wall, land, mesh1, foundation, mesh2
    global has_bones

    time1 = Blender.sys.time()  #for timing purposes

    # it would be preferable to move this to a class, so that we could
    # more simply enable/disable this.
    meshlog = open (filename + '.export.log', 'w')

    image_list = []
    for i in Image.Get():
       image_list.append(i.getFilename())
    
    group_classes = {}
    mesh_list = {}
    mesh_flags = [0,0,0,0,0,0,0,0]
    has_bones = False
    if Armature.Get() != {}:
        mesh_flags[0] = 1
        has_bones = True

    binding = None
    floor = None
    wall = None
    land = None
    mesh1 = None
    foundation = None
    mesh2 = None
    transform_list = []
    submesh_list = []
    room_list = {}
    control_list = []
    meshlog.write('>>> Starting Export')
    for m in Blender.Object.Get():
        if not m.getType() == "Mesh":
            continue

#        print '>>> "%s"' % (m.getData().getProperty("RemoveHead"))

        # Either-or. The import script sets the name of the Datagroup, but
        # originally here we were looking at the Mesh group itself.
        # Changed to datablock as that's what the import script affects
        n = m.getData().name
        meshlog.write('> Exporting Mesh: "%s"' % (n))
        if n.endswith('.binding'):
            binding = m
            mesh_flags[2] |= 1
            control_list.append(m)
        elif n.endswith('.floor'):
            floor = m
            mesh_flags[6] |= 1
            control_list.append(m)
        elif n.endswith('.wall'):
            wall = m
            mesh_flags[6] |= 8
            control_list.append(m)
        elif n.endswith('.land'):
            land = m
            mesh_flags[5] |= 2
            control_list.append(m)
        elif n.endswith('.mesh1'):
            mesh1 = m
            mesh_flags[6] |= 4
            control_list.append(m)
        elif n.endswith('.foundation'):
            foundation = m
            mesh_flags[5] |= 4
            control_list.append(m)
        elif n.endswith('.mesh2'):
            mesh2 = m
            mesh_flags[6] |= 16
            control_list.append(m)
        elif n.endswith('.room'):
            mesh_flags[6] |= 64
            room_list[n] = m

        # add alternative room selection, to match what we set in the import.
        elif room_mesh.match(n):
            mesh_flags[6] |= 64
            room_list[n] = m

        else:
            print '--- ' + m.name
            # grouped_mesh is looking for something in the format of *.00.*, 
            # such as blade_ca_.00.001 

            r = grouped_mesh.match(m.name)
            if r:
                group_class, groupid = r.groups()
                group_classes[groupid] = group_class
                if mesh_list.has_key(groupid):
                    mesh_list[groupid].append(m)
                else:
                    mesh_list[groupid] = [m]
            else:
                r = numbered_mesh.match(m.name)
                if r:
                    mesh_name = r.groups()[0]
                    if mesh_list.has_key(mesh_name):
                        mesh_list[mesh_name].append(m)
                    else:
                        mesh_list[mesh_name] = [m]
                else:
                    if mesh_list.has_key(m.name):
                        mesh_list[m.name].append(m)
                    else:
                        mesh_list[m.name] = [m]
            mats = m.getData().getMaterials()
            if mats:
                mat = mats[0]
                if mat.name not in material_keys:
                    material_list.append(mat)
                    material_keys.append(mat.name)
    
    #*******************************************
    # write header
    #*******************************************
    stream = open(filename, 'wb')
    stream.write(struct.pack("L",10))
    
    # ************ write image count *********** 
    stream.write(struct.pack("L", len(image_list)))
    meshlog.write('\n\n---> msh_header')
    meshlog.write('\nImages: %d' % (len(image_list)))

    # *********** write material count *********
    stream.write(struct.pack("L", len(material_keys)))
    meshlog.write('\nMaterials: %d' % (len(material_keys)))

    # ********** write mesh count *************
    stream.write(struct.pack("L",len(mesh_list)))
    meshlog.write('\nGroups: %d' % (len(mesh_list)))


    # Some of the flags in the format of a mesh are not readily 
    # with my current knowledge of blender. Have looked at a few of
    # the other formats, and think a dialog like the DirectX scripts
    # could go some way toward giving a user the flags they want. 
    #
    # Other options include the ability to add a Property in the logic
    # section of Blender -- maybe that's what we should be using, rather
    # than setting names. It does raise the bar a little, but shouldn't
    # be a problem. It does look like we could use 'GetAllProperties' on
    # a mesh, to let us do all that stuff, and even better, we could also
    # change the import script to set all this. No reason why we can't
    # move ahead ;)
    #
    # Doing that will definitely help with doing a blender->mesh structure. 
    # I suppose that would also give us the means to do all mesh-related
    # tweaks inside blender.

    # this seems to be deform_gorups
    if group_classes != []:
        # this is really unknown1 in original
        mesh_flags[7] = 1
#        mesh_flags[6] |= 128
#        mesh_flags[6] |= 2

    if submesh_list != {}:
        mesh_flags[6] |= 128
        
    if transform_list != []:
        mesh_flags[6] |= 2
    
    # ************* write flags ***************
    for flag in mesh_flags:
        meshlog.write('\n   Field: %d' % (flag))
        stream.write(struct.pack("B", flag))
        
    if transform_list != []:
        # ******* write transform count *******
        stream.write(struct.pack("L", len(transform_list)))
        meshlog.write('\nTransform: %d' % (len(transform_list)))

    if submesh_list != []:
        # ******** write submesh count ********
        stream.write(struct.pack("L", len(submesh_list)))
        meshlog.write('\nSubmesh: %d' % (len(submesh_list)))

    if room_list != []:
        # ********** write room count *********
        stream.write(struct.pack("L", len(room_list)))
        meshlog.write('\nRooms: %d' % (len(room_list)))

    meshlog.write('\n<--- msh_header\n\n')

    for i in image_list:
        fn = os.path.basename(i).replace(".png",".dds")
        write_nts(fn, stream)

    for mat in material_list:
        write_material(mat, stream)

    meshlog.write('\n>> Control Meshes')

    # Removed old method of doing control meshes, since they could
    # be added to the control_list in the wrong order, especially
    # with the creation of new sets.

    spos = stream.tell()
    meshlog.write('\n>> Marker Pre-CMesh: "%s"' % (spos))
    write_control_meshes(stream)

    spos = stream.tell()
    meshlog.write('\n>> Marker Pre-Roomlist: "%s"' % (spos))
    meshlog.write('\n>> Room List "%s"' % room_list)
    for n in room_list:
        write_nts(n, stream)
        write_basic_mesh(room_list[n], stream)

    spos = stream.tell()
    meshlog.write('\n>> Marker Pre-MainMesh: "%s"' % (spos))
    for g in mesh_list:
        write_group(mesh_list[g], stream)

    # bones goes here
    spos = stream.tell()
    meshlog.write('\n>> Marker Pre-Bones: "%s"' % (spos))
    write_armature(stream)

    spos = stream.tell()
    meshlog.write('\n>> Marker Pre-Groups: "%s"' % (spos))
    # These changes were suggested by Reacher@TMWS. Mark has said these
    # are not correct, on the basis of the method used for exporting
    # groups being incorrect, though I think this may plug a hole for 
    # the short term. We'll try one method, and if it doesn't work, use
    # the original as a fallback.
    #
    # This shouldn't interfere with the rest of the changes for sets, since
    # that is all based around control meshes.
    for g in mesh_list:
        meshlog.write('\nGroups: "%s"' % (g))
        try:
            write_nts(group_classes[g], stream)
        except:
            write_nts(g, stream)
    meshlog.write('\nGroup Classes: "%s"' % (group_classes))

#    if header.has_deform_groups or header.unknown1: 
#        name_trans = {}
#        for groupid in range(header.group_count):
#            group_name = "Group%02d"%groupid
#            name_trans[group_name] = read_nts(stream)
#        for mn in object_names:
#            gn, tail = mn.split(".",1)
#            if name_trans.has_key(gn):
#                mesh = Blender.Object.Get(mn)
#                new_name = "%s.%s.%s"%(name_trans[gn], gn[-2:], tail)
#                mesh.setName(new_name)


        
    stream.close()
    meshlog.close()

    print 'finished exporting: "%s" in %.4f sec.' % (filename, (Blender.sys.time()-time1))

#***********************************************
# register callback
#***********************************************
def my_callback(filename):
    save_msh(filename)

Blender.Window.FileSelector(my_callback, "Export MSH", '*.msh')
